/*-------------------------------------------------------------
 *  Copyright (c) 2010, MSS ComBlock. All rights reserved.
 *
 * 	Filename: iap.c
 *  Last modified: Jun 25, 2010
 *  Authors:
 *  	Alain Zarembowitch / MSS ComBlock
 *  Version: 1.0
 *  Inheritance: NXP secondary bootloader example. sbl_iap.c/sbl_iap.h/sbl_config.h
 *  Description: In-Application Programming
 *  Erase/write operation on the on-chip flash memory, as directed by the user application code.
 *  Includes utilities to save a ComBlock configuration structure (config) into the last flash block and retrieve it at power up.
 *
 *  Dependencies/Pre-requisite: N/A
 -------------------------------------------------------------*/
#include "main.h"
#include "iap.h"
#include "initConfig.h"

unsigned long command[5];
unsigned long result[5];
typedef void (*IAP)(unsigned long[], unsigned long[]);
IAP iap_entry;

// See Chapter 32: LPC17xx Flash memory interface and programming, Section 8: IAP
// for a detailed explanation of iap_entry

// Prepare sector(s) for write operation
void prepare_sector(unsigned start_sector, unsigned end_sector) {
	command[0] = 50; // command code
	command[1] = start_sector;
	command[2] = end_sector;
	iap_entry = (IAP) IAP_LOCATION;
	iap_entry(command, result);
}

// IAP Copy RAM to Flash command
void copy_Ram2Flash(unsigned long dest_addr, unsigned long ram_src_addr,
		unsigned long nBytes) {
	command[0] = 51; // command code
	command[1] = dest_addr; // must be multiple of 256B
	command[2] = ram_src_addr; // must be word boundary
	command[3] = nBytes; // 256/512/1024/4096
	command[4] = cpuFreq / 1000;
	iap_entry = (IAP) IAP_LOCATION;
	// test test test
	if((ram_src_addr & 0x00000003) != 0){
		toggle1(10);
	}
	// test test test
	FIO1SET = 0x00000002;
	iap_entry(command, result);
	// test test test
	FIO1CLR = 0x00000002;
}

// IAP Erase Sector(s) command
void erase_sector(unsigned long start_sector, unsigned long end_sector) {
	command[0] = 52; // command code
	command[1] = start_sector;
	command[2] = end_sector;
	command[3] = cpuFreq / 1000;
	iap_entry = (IAP) IAP_LOCATION;
	iap_entry(command, result);
}

// Blank check sector(s)
void blank_check(unsigned long start_sector, unsigned long end_sector) {
	command[0] = 53; // command code
	command[1] = start_sector;
	command[2] = end_sector;
	iap_entry = (IAP) IAP_LOCATION;
	iap_entry(command, result);
}

// Read part identification number
void read_partID(void) {
	command[0] = 54; // command code
	iap_entry = (IAP) IAP_LOCATION;
	iap_entry(command, result);
}

// Read Boot Code version number
void read_bootCodeVersion(void) {
	command[0] = 55; // command code
	iap_entry = (IAP) IAP_LOCATION;
	iap_entry(command, result);
}

// Read device serial number
void read_deviceSerialNo(void) {
	command[0] = 58; // command code
	iap_entry = (IAP) IAP_LOCATION;
	iap_entry(command, result);
}

// Compare <address1> <address2> <no of bytes>
void compare(unsigned long addr1, unsigned long addr2, unsigned long nBytes) {
	command[0] = 56; // command code
	command[1] = addr1; // must be word bondary
	command[2] = addr2; // must be word boundary
	command[3] = nBytes; // multiple of 4
	iap_entry = (IAP) IAP_LOCATION;
	iap_entry(command, result);
}

// Re-invoke ISP
void reinvoke_ISP(void) {
	command[0] = 57; // command code
	iap_entry = (IAP) IAP_LOCATION;
	iap_entry(command, result);
}

unsigned long start_address(unsigned long sectorNo) {
	if (sectorNo >= NSECTORS) {
		return 0; // out of range
	} else if (sectorNo < 16) {
		// 4KB sectors
		return sectorNo * 0x00001000;
	} else {
		// 32kB sectors
		return 0x00010000 + (sectorNo - 16) * 0x00008000;
	}
}

unsigned long end_address(unsigned long sectorNo) {
	if (sectorNo >= NSECTORS) {
		return 0; // out of range
	} else if (sectorNo < 16) {
		// 4KB sectors
		return ((sectorNo * 0x00001000) + 0x00000FFF);
	} else {
		// 32kB sectors
		return (0x00017FFF + (sectorNo - 16) * 0x00008000);
	}
}

/*
long load_configuration(void) {
	// read ComBlock configuration from flash sector CONFIG_SECTOR
	// and update the configuration structure
	unsigned long rptr;
	rptr = start_address(CONFIG_SECTOR); // configurations are stored in reserved sector CONFIG_SECTOR


	// read the hardware configuration first
	memcpy(&configHw, (void *) (rptr), sizeof(configHw));
	rptr += sizeof(configHw);

	// check if this is the first time the board is programmed
	if (configHw.default_sw_config == 0xFFFFFFFF) {
		// no previous configuration saved. Initialize with default.
		configHw.default_sw_config = 0;
		configHw.save_request_pending = 0;
	}

	// move to the start of the default software configuration
	rptr += configHw.default_sw_config * sizeof(configSw);
	// read the software configuration
	memcpy(&configSw, (void *) (rptr), sizeof(configSw));
	// make a copy of the non-volatile regs to the volatile registers
	memcpy(regv, &configSw.reg, sizeof(configSw.reg));
	return 0;
}
*/

long save_configuration(int sw_config_no, struct config_sw * csptr) {
	// save hardware + active software + optional alternate software configuration cs structures
	// into flash sector CONFIG_SECTOR (4KB max)
	// If sw_config_no < 0, save only the hardware + default software configurations (no alternate software configuration, cs is undefined)
	unsigned long rptr, wptr;
	char cache[4096];

	// move configurations to cache (RAM) before erasing the flash sector
	rptr = start_address(CONFIG_SECTOR); // configurations are stored in reserved sector CONFIG_SECTOR
	memcpy(&cache, (void *) (rptr), sizeof(cache));

	// update the hardware config in cache (if it changed)
	if (configHw.save_request_pending) {
		// clear the save request flags
		configHw.save_request_pending = 0;
		memcpy(&cache[0], &configHw, sizeof(configHw));
	}
	// update the default software config in cache (if it changed)
	if (configSw.save_request_pending) {
		// clear the save request flags
		configSw.save_request_pending = 0;
		wptr = sizeof(configHw) + (active_sw_config * sizeof(configSw));
		// make sure we do not write past the end of the buffer
		if ((wptr + sizeof(configSw)) > sizeof(cache))
			return -IAP_SRCTOOLARGE;
		memcpy(&cache[wptr], &configSw, sizeof(configSw));
	}
	// update the alternate software configuration in cache (when requested)
	if ((sw_config_no >= 0) && (sw_config_no < NSWCONFIGS)){
		wptr = sizeof(configHw) + (sw_config_no * sizeof(configSw));
		// make sure we do not write past the end of the buffer
		if ((wptr + sizeof(configSw)) > sizeof(cache))
			return -IAP_SRCTOOLARGE;
		memcpy(&cache[wptr], csptr, sizeof(configSw));
	}

	// disable interrupts;
	interruptsEnable(0);
	prepare_sector(CONFIG_SECTOR, CONFIG_SECTOR); // prepare sector
	if (result[0])
		return -result[0]; // error
	erase_sector(CONFIG_SECTOR, CONFIG_SECTOR); // erase sector
	if (result[0])
		return -result[0]; // error
	prepare_sector(CONFIG_SECTOR, CONFIG_SECTOR); // prepare sector
	if (result[0])
		return -result[0]; // error
	copy_Ram2Flash(start_address(CONFIG_SECTOR), (unsigned long) (&cache),
			sizeof(cache));
	if (result[0])
		return -result[0]; // error
	// enable interrupts
	interruptsEnable(1);
	return 0;
}

long read_sn(int * sn) {
	// read the 16-byte serial number (unique among all LPC17xx)
	// answer is in sn[0-3]

	// disable interrupts;
	interruptsEnable(0);
	read_deviceSerialNo();
	if (result[0])
		return result[0]; // error
	sn[0] = result[1];
	sn[1] = result[2];
	sn[2] = result[3];
	sn[3] = result[4];
	// enable interrupts
	interruptsEnable(1);
	return 0;

}

void read_mid(int sw_config_no, char * mid) {
	// returns the module identification number for the specified configuration.
	// (the configuration can be currently active or inactive, don't care)
	unsigned long rptr;
	struct config_sw c;

	if((sw_config_no >= NSWCONFIGS) || 	(sw_config_no < 0 ))
		return;	// out of range
	rptr = start_address(CONFIG_SECTOR); // configurations are stored in reserved sector CONFIG_SECTOR
	rptr += sizeof(configHw);
	// move to the start of the software configuration of interest
	rptr += sw_config_no * sizeof(configSw);
	// read the software configuration into a temporary buffer
	memcpy(&c, (void *) (rptr), sizeof(c));
	// make a copy of the 6-character module identification
	memcpy(mid, &c.mid, sizeof(c.mid));
	return;
}

void read_sw_config(int sw_config_no, struct config_sw * csptr) {
	// read ComBlock software configuration from flash sector CONFIG_SECTOR and return it in cs
	// sw_config_no must be in the range 0 - NSWCONFIGS-1.
	unsigned long rptr;
	if((sw_config_no >= NSWCONFIGS) || 	(sw_config_no < 0 ))
		return;	// out of range
	rptr = start_address(CONFIG_SECTOR); // configurations are stored in reserved sector CONFIG_SECTOR
	// move to the start of the software configuration of interest
	rptr += sizeof(configHw);
	rptr += sw_config_no * sizeof(configSw);
	// read the software configuration
	memcpy(csptr, (void *) (rptr), sizeof(configSw));
	return;
}

void read_hw_config(struct config_hw * chptr){
	// read ComBlock configuration from flash sector CONFIG_SECTOR
	// and update the configuration structure
	unsigned long rptr;
	rptr = start_address(CONFIG_SECTOR); // configurations are stored in reserved sector CONFIG_SECTOR


	// read the hardware configuration first
	memcpy(chptr, (void *) (rptr), sizeof(configHw));
	return;
}

long initialize_configs(void){
	// Change the entire flash sector CONFIG_SECTOR from blank (all FFs) to all zeros.
	// The objective is to mimic the ComBlock previous microcontroller (Atmel AVR) with zeroed control registers.
	int allFFs;
	unsigned long i, rptr;
	char cache[4096];

	// disable interrupts;
	interruptsEnable(0);
	blank_check(CONFIG_SECTOR, CONFIG_SECTOR);
	interruptsEnable(1);
	if (result[0])
		return -result[0];	// sector is not blank (i.e not all FFs). Nothing to do.

	// move configurations to cache (RAM) before erasing the flash sector
	rptr = start_address(CONFIG_SECTOR); // configurations are stored in reserved sector CONFIG_SECTOR
	memcpy(&cache, (void *) (rptr), sizeof(cache));

	// first check if the sector is all FFs
	allFFs = 1;
	for(i = 0; i<sizeof(cache); i++){
		if(cache[i] != 0xFF){
			allFFs = 0;
			break;
		}
	}

	if(!allFFs)
		return IAP_CMD_SUCCESS;

	// clear cache
	for(i = 0; i<sizeof(cache); i++)
		cache[i] = 0;

	// disable interrupts;
	interruptsEnable(0);
	prepare_sector(CONFIG_SECTOR, CONFIG_SECTOR); // prepare sector
	if (result[0])
		return -result[0]; // error
	erase_sector(CONFIG_SECTOR, CONFIG_SECTOR); // erase sector
	if (result[0])
		return -result[0]; // error
	prepare_sector(CONFIG_SECTOR, CONFIG_SECTOR); // prepare sector
	if (result[0])
		return -result[0]; // error
	copy_Ram2Flash(start_address(CONFIG_SECTOR), (unsigned long) (&cache),
			sizeof(cache));
	if (result[0])
		return -result[0]; // error
	// enable interrupts
	interruptsEnable(1);
	return IAP_CMD_SUCCESS;
}

// erase internal flash sector.
long internalFlash_EraseSector(unsigned long sectNo){
	// returns return 0 on success, a negative error number otherwise.
	// disable interrupts;
	interruptsEnable(0);
	prepare_sector(sectNo, sectNo); // prepare sector
	if (result[0])
		return -result[0]; // error
	erase_sector(sectNo, sectNo); // erase sector
	if (result[0])
		return -result[0]; // error
	// enable interrupts
	interruptsEnable(1);
	return 0;
}

long internalFlash_GCS(unsigned long sectNo, unsigned long *checksum){
	// GCS = get checksum
	// Compute and return the 16-bit checksum for the specified internal flash block
	// sectNo range 0 - 29 [LPC1759]
	unsigned long start_addr, sectSize, addr, i;
	unsigned char pageBuf[1024];

	// first check block number is valid
	if (sectNo >= NSECTORS)
		return -IAP_INVALID_SECTOR;	// sector number out of range

	// block size depends on sectNo
	if(sectNo <  16){
		// lower blocks are 4KB
		sectSize = 4096;	// 4KB
	} else {
		// upper blocks are 32KB
		sectSize = 32768;	// 32KB
	}
	start_addr = start_address(sectNo);

	*checksum = 0;

	for(addr = start_addr; addr < start_addr + sectSize; addr += sizeof(pageBuf)){
		memcpy(&pageBuf[0], (void *) (addr), sizeof(pageBuf));
		for(i = 0; i<sizeof(pageBuf); i++){
			*checksum += ((unsigned long)(pageBuf[i])) & 0x000000FF;
		}
	}
	// truncate to 16-bit checksum
	*checksum &= 0x0000FFFF;

	return IAP_CMD_SUCCESS;
}

long internalFlash_write(unsigned long sectNo, unsigned long offset, char * buf , unsigned long nBytes){
	// writes buffer to a given sector (with an offset from the start of the sector).
	// returns return 0 on success, a negative error number otherwise.
	unsigned long dest_addr;
	dest_addr = start_address(sectNo) + offset;

	// disable interrupts;
	interruptsEnable(0);
	prepare_sector(sectNo, sectNo); // prepare sector
	if(!result[0])
		copy_Ram2Flash(dest_addr, (unsigned long) (&buf[0]), nBytes);
	// enable interrupts
	interruptsEnable(1);
	return -result[0];
}

