/*-------------------------------------------------------------
 *  Copyright (c) 2010-2015, MSS ComBlock. All rights reserved.
 *
 * 	Filename: main.h
 *  Last modified: 12/12/15
 *  Authors:
 *  	Alain Zarembowitch / MSS
 *  Version: 2
 *  Inheritance: N/A
 ------------------------------------------------------------*/

#ifndef MAIN_H_
#define MAIN_H_

#include "LPC17xx.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define NTIMERS 4
#define TIMER1FAST 100	// wait 1 seconds before saving the comblock configuration changes into flash (to consolidate multiple successive changes into one sector programming operation)
#define TIMER1SLOW 250	// wait 2.5 seconds before saving the comblock configuration into flash (to consolidate multiple successive changes into one sector programming operation)
#define TIMER2 500		// 5 second inactivity timer
#define NSWCONFIGS 1  // number of configurations stored in flash. Range 0 - (NSWCONFIGS-1)
#define PROTECTION 0         // when set, FPGA configuration 0 (boot sectors) cannot be erased or reprogrammed
#define NREGS 254	// number of configuration registers
#define MAX_STRING_SIZE 273     // maximum number of characters in messages
#define ARM_FW_REV "301111"      // hardware platform + firmware identification (type, option, rev)
#define DEBUGGING 0				// 1 during debugging, 0 during normal operation

// structures
// rx and tx messages
typedef struct _string {
	int config_no;
	char buf[MAX_STRING_SIZE];
	int rptr; // rptr = 0, wptr != 0 -> buffer has not yet been processed
	int wptr; // wptr = 0 -> buffer is empty
	int msg_complete; // message ends with \n true/false
	int portNo; // port number for source/destination: 1 = top, 2 = right, 3 = bottom, 4 = left (ComBlock rule)
} volatile string_struct, *pstring_struct;

// ComBlock configuration is divided into two structures: board-related and software (FPGA/ARM)-related.
struct config_hw {
	char save_request_pending; // true when user requested to save the configuration into flash memory (into the last flash sector #29)
	int default_sw_config; // default software (FPGA, ARM) configuration number (ranges from 0 to NSWCONFIGS-1)
	char aid[6]; // assembly identification 4 char type, 1 option, 1 rev
	char asn[10];	// assembly serial number. Up to 10 non-zero characters
};
struct config_sw {
	char save_request_pending; // true when user requested to save the configuration into flash memory (into the last flash sector #29)
	unsigned char reg[NREGS];// registers
	char mid[6]; // module identification, 4 char module type, 1 module option, 1 rev
	char protection; // 0 = unprotected, 1 = protected
	long fpga_start_address;
};

// variables
extern volatile unsigned long SysTickCnt; //SysTick Counter  (can be changed in ISR -> volatile)
extern unsigned long cpuFreq; // current CPU clock frequency in Hz
extern volatile unsigned long timer[NTIMERS];
// timers, expressed in multiple of the system tick (10ms). 0 means expired.
// volatile because the Systick ISR will modify the values
// timer[0] = NAND flash timer
// timer[1] = forced delay to save the config into flash
// timer[2] = inactivity timer
// timer[3] = misc.
extern volatile string_struct rxLine; // rx command line
extern volatile string_struct txLine; // tx command line
extern char address[4]; // ComBlock address, assigned dynamically (string of 3 characters + \0 end of string)
extern long commandPortNo; // port through which this board receives commands: 1 = top, 2 = right, 3 = bottom, 4 = left (ComBlock rule)
unsigned char forwardMask; // Mask output ports. bit0 = top port1, bit1 = right port 2, etc. '1' means the port is blocked, '0' means the port is enabled for output
extern struct config_hw configHw; // hardware-related configuration.
extern struct config_sw configSw; // software/FPGA-related configuration non-volatile
extern char regv[NREGS]; // control registers, volatile copy of configSw.reg[]
extern int active_sw_config;	// current active configuration (generally the same as default_sw_config, but it could be temporary different until the new FPGA config is loaded)
extern int usb_softconnect;	// 0 for no vbus/no connect, 1 for vbus/connected
extern unsigned long RF_gain;	// receiver RF gain (2 RF attenuators + 1 digital gain control)
extern unsigned long BB_gain;	// receiver baseband gain (1 digital gain control)
extern long RF_gain_too_small_duration; // below normal range samples timer
extern long RF_gain_too_large_duration; // above normal range samples timer
extern long fc;	// estimated receiver center frequency (needed to compute attenuators response)
extern unsigned long agc_in;	// gain control value from the 40-pin connector, low-pass filtered = AD0[5]
extern unsigned long rf_power_det;	// rf power detection from RF mixer = AD0[2]

// function prototypes
extern void initConfig(void); // system initialization
extern void ResetISR(void);

#endif /* MAIN_H_ */
