/*----------------------------------------------------------------------------
 *      U S B  -  K e r n e l
 *----------------------------------------------------------------------------
 *      Name:    USBDESC.C
 *      Purpose: USB Descriptors
 *      Version: V1.10
 *----------------------------------------------------------------------------
 *      This software is supplied "AS IS" without any warranties, express,
 *      implied or statutory, including but not limited to the implied
 *      warranties of fitness for purpose, satisfactory quality and
 *      noninfringement. Keil extends you a royalty-free right to reproduce
 *      and distribute executable files created using this software for use
 *      on NXP Semiconductors LPC family microcontroller devices only. Nothing
 *      else gives you the right to use this software.
 *
 *	 	User manual: http://infocenter.arm.com/help/topic/com.arm.doc.dui0374a/DUI0374A_rl_usb_user_guide.pdf
 *
 *
 *      Copyright (c) 2005-2009 Keil Software.
 *---------------------------------------------------------------------------*/
// customized for ComBlock platforms
// 12-10-10 AZ
// New feature: two descriptors depending on the flash_mode_jumper() jumper position
// Change product string (here COM-16xx) as needed
// LPC17xx imposes FIXED ENDPOINTS 2 and 5 for Bulk In/Out (thus the difference with previous ComBlock FPGA-based SIE).

#include "lpc_types.h"

#include "usb.h"
#include "msc.h"
#include "usbcfg.h"
#include "usbdesc.h"


/* USB Standard Device Descriptor */
const uint8_t USB_DeviceDescriptor[] = {
  USB_DEVICE_DESC_SIZE,              /* bLength */
  USB_DEVICE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */          /* bcdUSB */
  0x00,                              /* bDeviceClass */
  0x00,                              /* bDeviceSubClass */
  0x00,                              /* bDeviceProtocol */
  USB_MAX_PACKET0,                   /* bMaxPacketSize0 */
  WBVAL(0xC251),                     /* idVendor */
  WBVAL(0x1303),                     /* idProduct */
  WBVAL(0x0100), /* 1.00 */          /* bcdDevice */
  0x04,                              /* iManufacturer */
  0x20,                              /* iProduct */
  0x48,                              /* iSerialNumber */
  0x01                               /* bNumConfigurations */
};

/* USB Configuration Descriptor */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor */
const uint8_t USB_ConfigDescriptor[] = {
/* Configuration 1 */
  USB_CONFIGUARTION_DESC_SIZE,       /* bDescriptorType */
  USB_CONFIGURATION_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(                             /* wTotalLength */
    1*USB_CONFIGUARTION_DESC_SIZE +
    1*USB_INTERFACE_DESC_SIZE     +
    2*USB_ENDPOINT_DESC_SIZE
  ),
  0x01,                              /* bNumInterfaces */
  0x01,                              /* bConfigurationValue */
  0x00,                              /* iConfiguration */
  USB_CONFIG_BUS_POWERED /*|*/       /* bmAttributes */
/*USB_CONFIG_REMOTE_WAKEUP*/,
  USB_CONFIG_POWER_MA(100),          /* bMaxPower */
/* Interface 0, Alternate Setting 0, MSC Class */
  USB_INTERFACE_DESC_SIZE,           /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,     /* bDescriptorType */
  0x00,                              /* bInterfaceNumber */
  0x00,                              /* bAlternateSetting */
  0x02,                              /* bNumEndpoints */
  USB_DEVICE_CLASS_STORAGE,          /* bInterfaceClass */
  MSC_SUBCLASS_SCSI,                 /* bInterfaceSubClass */
  MSC_PROTOCOL_BULK_ONLY,            /* bInterfaceProtocol */
  0x62,                              /* iInterface */
/* Bulk In Endpoint */
  USB_ENDPOINT_DESC_SIZE,            /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,      /* bDescriptorType */
  USB_ENDPOINT_IN(2),                /* bEndpointAddress */
  USB_ENDPOINT_TYPE_BULK,            /* bmAttributes */
  WBVAL(0x0040),                     /* wMaxPacketSize */
  0,                                 /* bInterval */
/* Bulk Out Endpoint */
  USB_ENDPOINT_DESC_SIZE,            /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,      /* bDescriptorType */
  USB_ENDPOINT_OUT(2),               /* bEndpointAddress */
  USB_ENDPOINT_TYPE_BULK,            /* bmAttributes */
  WBVAL(0x0040),                     /* wMaxPacketSize */
  0,                                 /* bInterval */
/* Terminator */
  0                                  /* bLength */
};

/* USB String Descriptor (optional) */
const uint8_t USB_StringDescriptor[] = {
/* Index 0x00: LANGID Codes */
  0x04,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  WBVAL(0x0409), /* US English */    /* wLANGID */
/* Index 0x04: Manufacturer */
  0x1C,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'N',0,
  'X',0,
  'P',0,
  ' ',0,
  'S',0,
  'e',0,
  'm',0,
  'i',0,
  'c',0,
  'o',0,
  'n',0,
  'd',0,
  ' ',0,
/* Index 0x20: Product */
  0x28,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'N',0,
  'X',0,
  'P',0,
  ' ',0,
  'L',0,
  'P',0,
  'C',0,
  '1',0,
  '7',0,
  'x',0,
  'x',0,
  ' ',0,
  'M',0,
  'e',0,
  'm',0,
  'o',0,
  'r',0,
  'y',0,
  ' ',0,
/* Index 0x48: Serial Number */
  0x1A,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'D',0,
  'E',0,
  'M',0,
  'O',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
/* Index 0x62: Interface 0, Alternate Setting 0 */
  0x0E,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'M',0,
  'e',0,
  'm',0,
  'o',0,
  'r',0,
  'y',0,
};


/*============== ComBlock standard descriptors ============================== */

/* USB Standard Device Descriptor */
const uint8_t USB_DeviceDescriptor_CB[] = {
  0x12,             				/* size of this descriptor in bytes */
  0x01,        						/* DEVICE descriptor type */
  WBVAL(0x0200), 					/* USB specification release 2.00 */
  0xFF,                              /* Vendor-specific class, not registered with USB-IF */
  0xFF,                              /* Vendor-specific subclass, not registered with USB-IF */
  0xFF,                              /* Vendor-specific protocol class, not registered with USB-IF */
  0x40,                   			 /* Maximum packet size for endpoint 0 (64) */
  WBVAL(0x0000),                     /* Vendor ID */
  WBVAL(0x0004),                     /* Product ID */
  WBVAL(0x0100), /* 1.00 */          /* Device release number */
  0x04,                              /* Manufacturer string*/
  0x20,                              /* Product string */
  0x00,                              /* SerialNumber string */
  0x01                               /* Number of possible configurations at the current operating speed */
};

/* USB Configuration Descriptor */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor */
const uint8_t USB_ConfigDescriptor_CB[] = {
  /* Configuration 1 */
  0x09,       						 /* Size of this descriptor in bytes */
  0x02, 							 /* Configuration descriptor type */
  WBVAL(0x002E),                     /* Total length of data returned for this configuration (1 config + 1 interface descriptor + 4 endpoints) */
  0x01,                              /* Number of interfaces supported by this configuration */
  0x01,                              /* Configuration number */
  0x00,                              /* Index of string descriptor describing this configuration (no string) */
  0x80,								 /* could be self-powered or not. declare as usb-powered */
  0xFA,								 /* declare the maximum 500mA current drawn over USB */
  /* Interface descriptor 0 */
  0x09,           					 /* size of this descriptor in bytes */
  0x04,							     /* INTERFACE descriptor size */
  0x00,                              /* Number of this interface */
  0x00,                              /* Alternate Setting */
  0x04,                              /* Number of endpoints (excluding endpoint 0 default control pipe) */
  0xFF,          					 /* Interface Class code*/
  0xFF,          					 /* Interface Subclass code*/
  0xFF,          					 /* Interface protocol*/
  0x00,                              /* index of string descriptor */
  /* In Endpoint descriptor (device to host direction) */
  0x07,            					/* size of this descriptor in bytes */
  0x05,      						/* ENDPOINT descriptor type */
  0x82,							    /* IN */
  0x02,            					/* Attribute: Bulk, data endpoint */
  WBVAL(0x0040),                    /* Maximum Packet Size */
  0x00,                             /* No polling in this direction */
  /* Out Endpoint descriptor (host to device direction) */
  0x07,            					/* size of this descriptor in bytes */
  0x05,      						/* ENDPOINT descriptor type */
  0x02,							    /* OUT */
  0x02,            					/* Attribute: Bulk, data endpoint */
  WBVAL(0x0040),                    /* Maximum Packet Size */
  0x00,                             /* No polling in this direction */
  /* In Endpoint descriptor (device to host direction) */
  0x07,            					/* size of this descriptor in bytes */
  0x05,      						/* ENDPOINT descriptor type */
  0x85,							    /* IN */
  0x02,            					/* Attribute: Bulk, data endpoint */
  WBVAL(0x0040),                    /* Maximum Packet Size */
  0x00,                             /* No polling in this direction */
  /* Out Endpoint descriptor (host to device direction) */
  0x07,            					/* size of this descriptor in bytes */
  0x05,      						/* ENDPOINT descriptor type */
  0x05,							    /* OUT */
  0x02,            					/* Attribute: Bulk, data endpoint */
  WBVAL(0x0040),                    /* Maximum Packet Size */
  0x00,                             /* No polling in this direction */
  /* Terminator */
    0                                  /* bLength */
};

/* USB String Descriptor (optional) */
const uint8_t USB_StringDescriptor_CB[] = {
/* Index 0x00: LANGID Codes */
  0x04,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  WBVAL(0x0409), /* US English */    /* wLANGID */
/* Index 0x04: Manufacturer */
  0x1C,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'M',0,
  'S',0,
  'S',0,
  ' ',0,
  'C',0,
  'o',0,
  'm',0,
  'b',0,
  'l',0,
  'o',0,
  'c',0,
  'k',0,
  ' ',0,
/* Index 0x20: Product */
  0x28,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'C',0,
  'O',0,
  'M',0,
  '-',0,
  '3',0,
  '0',0,
  '1',0,
  '1',0,
  ' ',0,
  ' ',0,
  ' ',0,
  ' ',0,
  ' ',0,
  ' ',0,
  ' ',0,
  ' ',0,
  ' ',0,
  ' ',0,
  ' ',0,
/* Index 0x48: Serial Number */
  0x1A,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  '3',0,
  '0',0,
  '1',0,
  '1',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
  '0',0,
/* Index 0x62: Interface 0, Alternate Setting 0 */
  0x0E,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'C',0,
  'C',0,
  'C',0,
  ' ',0,
  ' ',0,
  ' ',0,
};
